@@ -146,9 +146,13 @@ long numa_node_size(int node, long *freep);
 
 int numa_pagesize(void);
 
-/* Set with all nodes. Only valid after numa_available. */
+/* Set with all nodes from which the calling process may allocate memory.
+   Only valid after numa_available. */
 extern struct bitmask *numa_all_nodes_ptr;
 
+/* Set with all nodes the kernel has exposed to userspace */
+extern struct bitmask *numa_nodes_ptr;
+
 /* For source compatibility */
 extern nodemask_t numa_all_nodes;
 
@@ -208,6 +212,8 @@ void *numa_alloc_onnode(size_t size, int node);
 void *numa_alloc_local(size_t size);
 /* Allocation with current policy */
 void *numa_alloc(size_t size);
+/* Change the size of a memory area preserving the memory policy */
+void *numa_realloc(void *old_addr, size_t old_size, size_t new_size);
 /* Free memory allocated by the functions above */
 void numa_free(void *mem, size_t size);
 
@@ -256,9 +262,11 @@ int numa_num_configured_cpus();
 
 /* maximum cpus allowed to current task */
 int numa_num_task_cpus();
+int numa_num_thread_cpus(); /* backward compatibility */
 
 /* maximum nodes allowed to current task */
 int numa_num_task_nodes();
+int numa_num_thread_nodes(); /* backward compatibility */
 
 /* allocate a bitmask the size of the kernel cpumask_t */
 struct bitmask *numa_allocate_cpumask();
@@ -302,10 +310,18 @@ int numa_sched_getaffinity(pid_t, struct bitmask *);
 int numa_sched_setaffinity(pid_t, struct bitmask *);
 
 /* Convert an ascii list of nodes to a bitmask */
-struct bitmask *numa_parse_nodestring(char *);
+struct bitmask *numa_parse_nodestring(const char *);
+
+/* Convert an ascii list of nodes to a bitmask without current nodeset
+ * dependency */
+struct bitmask *numa_parse_nodestring_all(const char *);
 
 /* Convert an ascii list of cpu to a bitmask */
-struct bitmask *numa_parse_cpustring(char *);
+struct bitmask *numa_parse_cpustring(const char *);
+
+/* Convert an ascii list of cpu to a bitmask without current taskset
+ * dependency */
+struct bitmask *numa_parse_cpustring_all(const char *);
 
 /*
  * The following functions are for source code compatibility
